/****************************************************************************
** Meta object code from reading C++ file 'plugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "plugin.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14OAuth2PluginNS6PluginE_t {};
} // unnamed namespace

template <> constexpr inline auto OAuth2PluginNS::Plugin::qt_create_metaobjectdata<qt_meta_tag_ZN14OAuth2PluginNS6PluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OAuth2PluginNS::Plugin",
        "type",
        "",
        "mechanisms",
        "cancel",
        "process",
        "SignOn::SessionData",
        "inData",
        "mechanism",
        "userActionFinished",
        "SignOn::UiSessionData",
        "data",
        "refresh"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'type'
        QtMocHelpers::SlotData<QString() const>(1, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'mechanisms'
        QtMocHelpers::SlotData<QStringList() const>(3, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'cancel'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'process'
        QtMocHelpers::SlotData<void(const SignOn::SessionData &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'process'
        QtMocHelpers::SlotData<void(const SignOn::SessionData &)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'userActionFinished'
        QtMocHelpers::SlotData<void(const SignOn::UiSessionData &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'refresh'
        QtMocHelpers::SlotData<void(const SignOn::UiSessionData &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Plugin, qt_meta_tag_ZN14OAuth2PluginNS6PluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OAuth2PluginNS::Plugin::staticMetaObject = { {
    QMetaObject::SuperData::link<AuthPluginInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OAuth2PluginNS6PluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OAuth2PluginNS6PluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14OAuth2PluginNS6PluginE_t>.metaTypes,
    nullptr
} };

void OAuth2PluginNS::Plugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Plugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->type();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 1: { QStringList _r = _t->mechanisms();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->cancel(); break;
        case 3: _t->process((*reinterpret_cast<std::add_pointer_t<SignOn::SessionData>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->process((*reinterpret_cast<std::add_pointer_t<SignOn::SessionData>>(_a[1]))); break;
        case 5: _t->userActionFinished((*reinterpret_cast<std::add_pointer_t<SignOn::UiSessionData>>(_a[1]))); break;
        case 6: _t->refresh((*reinterpret_cast<std::add_pointer_t<SignOn::UiSessionData>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SignOn::SessionData >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SignOn::SessionData >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SignOn::UiSessionData >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SignOn::UiSessionData >(); break;
            }
            break;
        }
    }
}

const QMetaObject *OAuth2PluginNS::Plugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OAuth2PluginNS::Plugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OAuth2PluginNS6PluginE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "com.nokia.SingleSignOn.PluginInterface/1.3"))
        return static_cast< AuthPluginInterface*>(this);
    return AuthPluginInterface::qt_metacast(_clname);
}

int OAuth2PluginNS::Plugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AuthPluginInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
